#if !defined( ISR_INTERFACE_SUN5T_H )
#define ISR_INTERFACE_SUN5T_H

/*
   ISR Interface Example.
  
   This include file declares an ISR function, an ISR data function, and two
   global variables for shared access by the ISR and the model.
  
   The interrupt being used for this example is for a Solaris itimer (interval
   timer).  See the Solaris man page for setitimer() for details on how
   they work.
  
   This file should be included by the ObjecTime model that registers
   the ISR and accesses the global variables.
  
   There are many better ways for user applications (models) and ISRs to share
   and transfer data, but global variables were chosen for convenience of
   creating this example.  In a REAL application, a thread/interrupt safe
   resource, such as an RTOS queue, MUST be chosen.
*/
  
#if !defined( MAKE_EXTERN )
    #define MAKE_EXTERN extern
#endif

#include <semaphore.h>

/*
   Global variable used by the ISR to leave an indication to the application
   that the ISR has placed something in some location (ISRCounter for
   this example).  The ISR sets it; the application (processing function of
   the ISR layer actor) clears it.
*/
MAKE_EXTERN int ISRFired;

/*
*  Global variable representing some location for the ISR to leave data for
*  the application to pick up.
*/
MAKE_EXTERN int ISRCounter;


/*
*  Function for registering data for ISR to use.  The ISR will wake up the
*  actor (blocked on the registered external interface function) if it has
*  determined that the actor may be sleeping.  For this example, when
*  ISR_Fired == 0 it is possible that the actor is sleeping so
*  ISR_Fired will be assigned 1 and the controller
*  will be woken up.  When ISR_Fired == 1, then it is assumed that the
*  controller is not sleeping because it would have been woken up by a
*  previous call of the ISR.
MAKE_EXTERN void registerWakeupWithISR( sem_t *sem );


/*
*  The actual ISR.
* 
*  This ISR is called when an itimer expires.
*/
void anISR( void );

#endif    // ISR_INTERFACE_SUN5T_H
