/* tcpServer.c - TCP server example */


/*
DESCRIPTION
This file contains the server-side of the VxWorks TCP example code.
The example code demonstrates the useage of several BSD 4.3-style
socket routine calls.
*/

/* includes */


#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#define SERVER_PORT_NUM 35555
#define SERVER_MAX_CONNECTIONS 5

/****************************************************************************
*
* tcpServer - accept and process requests over a TCP socket
*
* This routine creates a TCP socket, and accepts connections over the socket
* from clients. Each client connection is handled by spawning a separate
* task to handle client requests.
*
*
* RETURNS: Never, or -1 if a resources could not be allocated.
*/

int main( int argc, char *argv[] ) {
	
struct sockaddr_in  serverAddr;    /* server's socket address */

struct sockaddr_in  clientAddr;    /* client's socket address */
int                 sockAddrSize;  /* size of socket address structure */
int                 sFd;           /* socket file descriptor */
int                 newFd;         /* socket descriptor from accept */
int                 ix = 0;        /* counter for work task names */
/* set up the local address */



sockAddrSize = sizeof (struct sockaddr);
bzero ((char *) &serverAddr, sockAddrSize);
serverAddr.sin_family = AF_INET;
serverAddr.sin_port = htons (SERVER_PORT_NUM);
serverAddr.sin_addr.s_addr = htonl (INADDR_ANY);

/* create a TCP-based socket */
if ((sFd = socket (AF_INET, SOCK_STREAM, 0)) == -1) {
	perror ("socket");
	return (-1);
}

/* bind socket to local address */
if (bind (sFd, (struct sockaddr *) &serverAddr, sockAddrSize) == -1) {
	perror ("bind");
	close (sFd);
	return (-1);
}

/* create queue for client connection requests */
if (listen (sFd, SERVER_MAX_CONNECTIONS) == -1) {
	perror ("listen");
	close (sFd);
	return (-1);
}

/* accept new connect requests and spawn process to handle them */
for( ; ; ) {
    if ((newFd = accept (sFd, (struct sockaddr *) &clientAddr,
		&sockAddrSize)) == -1) {
		perror ("accept");
		close (sFd);
		return (-1);
    }

    printf ( "new connection accepted!\n");

    if (fork() == 0 ) {
       char socketBuffer[256];
       char sendBuffer[256];
       int  rc;

       /* we are child process */
       printf ("New process created. socket = %d\n", newFd );
       for (;;) {
	   memset( socketBuffer, 0, 255 );
           rc = recv( newFd, socketBuffer, 255, 0 ); 
           if( rc == -1) {
	      perror( "recv" );
	      exit( -1 );
           } else if ( rc > 0 ) {
              printf ("%s - thank you!\n", socketBuffer );
	      sprintf( sendBuffer, "%s received\n", socketBuffer );
	      rc = send( newFd, sendBuffer, strlen( sendBuffer ) + 1, 0 );
	      if (rc < 0) {
		  printf ("send failed!\n");
		  exit( -1 );
              }
           }
       }
    }

}

}


